@@ -27,7 +27,7 @@ from string import Template
 import pyppyn
 from gravitybee.distutils_utils import fix_distutils
 
-__version__ = "0.1.86"
+__version__ = "0.3.0"
 EXIT_OKAY = 0
 EXIT_NOT_OKAY = 1
 FILE_DIR = ".gravitybee"
@@ -171,6 +171,14 @@ class Arguments():
             logger.error("work_dir must not exist. It may be deleted.")
             raise FileExistsError
 
+        self.flags["include_setup_extras"] = kwargs.get(
+            'include_setup_extras',
+            os.environ.get(
+                'GB_INCLUDE_SETUP_EXTRAS',
+                False
+            )
+        )
+
         self.flags["one_dir"] = kwargs.get(
             'onedir',
             os.environ.get(
@@ -231,6 +239,17 @@ class Arguments():
             )
         )
 
+        self.info["hook_template"] = kwargs.get(
+            'hook_template',
+            os.environ.get(
+                'GB_HOOK_TEMPLATE',
+                os.path.join(
+                    os.path.dirname(__file__),
+                    "hook-template"
+                )
+            )
+        )
+
         pl_sys = platform.system().lower()
         self.info["operating_system"] = pl_sys if pl_sys != 'darwin' else 'osx'
 
@@ -254,9 +273,14 @@ class Arguments():
         logger.info("clean: %s", self.flags["clean"])
         logger.info("work_dir: %s", self.directories["work"])
         logger.info("onedir: %s", self.flags["one_dir"])
+        logger.info(
+            "include_setup_extras: %s",
+            self.flags["include_setup_extras"]
+        )
         logger.info("staging_dir: %s", self.directories["staging"])
         logger.info("with_latest: %s", self.flags["with_latest"])
         logger.info("sha: %s", self.info["sha"])
+        logger.info("hook_template: %s", self.info["hook_template"])
 
         if self.extra["data"] is not None:
             for extra_data in self.extra["data"]:
@@ -450,39 +474,52 @@ class PackageGenerator():
         logger.info("Package generator:")
         logger.info("standalone_name: %s", self.standalone_name)
 
-    def _create_hook(self):
+    def _generate_hook_text_from_template(self):
         # get the hook ready
-        template = Template(
-            open(
-                os.path.join(
-                    self.gb_dir,
-                    "hook-template"
-                ),
-                "r"
-            ).read()
-        )
+        with open(
+            os.path.join(self.gb_dir, "hook-template"), "r", encoding="utf8"
+        ) as hook_fh:
+            template = Template(hook_fh.read())
 
         hook = template.safe_substitute(
-            {'app_name': self.args.info["app_name"]})
+            {
+                'app_name': self.args.info["app_name"],
+                'pkg_name': self.args.info["pkg_name"]
+            })
 
         # 1 - extra data
         hook += "# collection extra data, if any (using --extra-data option)"
         for data in self.args.extra.get("data") or []:
+            extradir = self.args.directories["pkg"] + os.sep
+            if self.args.directories["src"] != ".":
+                extradir += self.args.directories["src"] + os.sep
+            extradir += self.args.info["pkg_name"] + os.sep + data
+            # handle relative paths for extra data
+            pkg = os.path.normpath(
+                self.args.info["pkg_name"] + "/" + data
+            ).replace("\\", "/")
+            # Normalize the path, replace backslashes with forward slashes
+            # Otherwise, we risk character literals (like \t) being in the
+            # path string
+            extradir = os.path.normpath(extradir).replace("\\", "/")
             hook += "\ndatas.append(('"
-            hook += self.args.directories["pkg"] + os.sep
-            if self.args.directories["src"] != '.':
-                hook += self.args.directories["src"] + os.sep
-            hook += self.args.info["pkg_name"] + os.sep + data
-            hook += "', '" + self.args.info["pkg_name"] + "/" + data + "'))"
+            hook += extradir
+            hook += "', '" + pkg + "'))"
             hook += "\n\n"
 
         # 2 - package metadata
         hook += "# add dependency metadata"
-        for package in self.args.pyppy.get_required():
+        for package in self.args.pyppy.get_required(
+            self.args.flags["include_setup_extras"]
+        ):
             # datas += copy_metadata(pkg)
             hook += "\ndatas += copy_metadata('" + package + "')"
 
         hook += "\n"
+        return hook
+
+    def _create_hook(self):
+        hook = self._generate_hook_text_from_template()
 
         # 3 - write file
         self.files["hook"] = os.path.join(
@@ -490,7 +527,7 @@ class PackageGenerator():
             'hooks',
             "hook-" + self.args.info["pkg_name"] + ".py"
         )
-        with open(self.files["hook"], "w+") as hook_file:
+        with open(self.files["hook"], "w+", encoding="utf8") as hook_file:
             hook_file.write(hook)
 
         logger.info("Created hook file: %s", self.files["hook"])
@@ -523,7 +560,9 @@ class PackageGenerator():
 
                 logger.info("SHA256 hash file: %s", self.files["sha"])
 
-                with open(self.files["sha"], 'w') as sha_file:
+                with open(
+                    self.files["sha"], 'w', encoding="utf8"
+                ) as sha_file:
                     sha_file.write(json.dumps(sha_dict))
 
     def _stage_artifacts(self):
@@ -649,6 +688,9 @@ class PackageGenerator():
         gb_info['clean'] = self.args.flags["clean"]
         gb_info['work_dir'] = self.args.directories["work"]
         gb_info['onedir'] = self.args.flags["one_dir"]
+        gb_info['include_setup_extras'] = self.args.flags[
+            "include_setup_extras"
+        ]
         gb_info['staging_dir'] = self.args.directories["staging"]
         gb_info['with_latest'] = self.args.flags["with_latest"]
         gb_info['gen_file'] = self.files["gen"]
@@ -674,7 +716,9 @@ class PackageGenerator():
             "Writing information file: %s",
             PackageGenerator.INFO_FILE
         )
-        with open(PackageGenerator.INFO_FILE, 'w') as info_file:
+        with open(
+            PackageGenerator.INFO_FILE, 'w', encoding="utf8"
+        ) as info_file:
             info_file.write(json.dumps(gb_info))
 
         return gb_info
@@ -732,7 +776,9 @@ class PackageGenerator():
                 "Writing files file: %s",
                 PackageGenerator.FILES_FILE
             )
-            with open(PackageGenerator.FILES_FILE, 'w') as file_file:
+            with open(
+                PackageGenerator.FILES_FILE, 'w', encoding="utf8"
+            ) as file_file:
                 file_file.write(json.dumps(gb_files))
 
             # ENVIRONS ----------------------------------------------
@@ -844,7 +890,9 @@ class PackageGenerator():
         for extra_package in list(
                 set(self.EXTRA_REQD_PACKAGES) | set(self.args.extra["pkgs"])
         ):
-            if extra_package not in self.args.pyppy.get_required():
+            if extra_package not in self.args.pyppy.get_required(
+                    self.args.flags["include_setup_extras"]
+            ):
                 pyppyn.ConfigRep.install_package(extra_package)
 
         for extra_module in list(
